@@ -1,3 +1,12 @@
+// Copyright (c) 2016 University of Helsinki                          
+//                                                                    
+// This library is free software; you can redistribute it and/or      
+// modify it under the terms of the GNU Lesser General Public         
+// License as published by the Free Software Foundation; either       
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more      
+// information.
+
 #include "HfstExceptionDefs.h"
 
 std::string hfst_exception;
@@ -40,6 +49,8 @@ HFST_EXCEPTION_CHILD_DEFINITION(HfstTransducerTypeMismatchException);
 
 HFST_EXCEPTION_CHILD_DEFINITION(ImplementationTypeNotAvailableException);
 
+HFST_EXCEPTION_CHILD_DEFINITION(FileIsInGZFormatException);
+
 HFST_EXCEPTION_CHILD_DEFINITION(FunctionNotImplementedException);
 
 HFST_EXCEPTION_CHILD_DEFINITION(StreamNotReadableException);
