@@ -1,3 +1,12 @@
+// Copyright (c) 2016 University of Helsinki                          
+//                                                                    
+// This library is free software; you can redistribute it and/or      
+// modify it under the terms of the GNU Lesser General Public         
+// License as published by the Free Software Foundation; either       
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more      
+// information.
+
 #ifndef _HFST_EXCEPTION_DEFS_H_
 #define _HFST_EXCEPTION_DEFS_H_
 
@@ -197,6 +206,8 @@ try {
 */
 HFST_EXCEPTION_CHILD_DECLARATION(NotTransducerStreamException);
 
+HFST_EXCEPTION_CHILD_DECLARATION(FileIsInGZFormatException);
+
 /** \brief The stream is not in valid AT&T format. 
 
     An example. The file "testfile.att" contains
