@@ -1,14 +1,11 @@
-//       This program is free software: you can redistribute it and/or modify
-//       it under the terms of the GNU General Public License as published by
-//       the Free Software Foundation, version 3 of the License.
-//
-//       This program is distributed in the hope that it will be useful,
-//       but WITHOUT ANY WARRANTY; without even the implied warranty of
-//       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//       GNU General Public License for more details.
-//
-//       You should have received a copy of the GNU General Public License
-//       along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// Copyright (c) 2016 University of Helsinki                          
+//                                                                    
+// This library is free software; you can redistribute it and/or      
+// modify it under the terms of the GNU Lesser General Public         
+// License as published by the Free Software Foundation; either       
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more      
+// information.
 
 //! @FIXME: The structure of this class and its functions is disorganised.
 
@@ -589,6 +586,22 @@ namespace hfst
       case '#':  // foma
         return FOMA_;
         break;
+      case (char)0x1f:  // native foma (gz magic number is 1F 8B 08)
+        {
+          char c0 = (char)stream_get();
+          if (stream_eof()) { HFST_THROW(EndOfStreamException); }
+          char c1 = (char)stream_get();
+          if (stream_eof()) { HFST_THROW(EndOfStreamException); }
+          char c2 = (char)stream_get();
+          if (stream_eof()) { HFST_THROW(EndOfStreamException); }
+          stream_unget(c2);
+          stream_unget(c1);
+          stream_unget(c0);
+          if (c0 == (char)0x1f && c1 == (char)0x8b && c2 == (char)0x08)
+            HFST_THROW(FileIsInGZFormatException);
+          else
+            HFST_THROW(NotTransducerStreamException);
+        }
       case 'a':  // SFST
         return SFST_;
         break;
@@ -930,15 +943,10 @@ namespace hfst
     bytes_to_skip(0), filename(std::string()), has_hfst_header(false),
     hfst_version_2_weighted_transducer(false)
   {
-    try { 
-      input_stream = &std::cin;
-      if (stream_eof())
-        HFST_THROW(EndOfStreamException);
-      type = stream_fst_type();
-    }
-
-    catch (const HfstException e)
-    { throw e; }
+    input_stream = &std::cin;
+    if (stream_eof())
+      HFST_THROW(EndOfStreamException);
+    type = stream_fst_type();
 
     if ( ! HfstTransducer::is_implementation_type_available(type)) {
       HFST_THROW(ImplementationTypeNotAvailableException);
@@ -1001,91 +1009,87 @@ namespace hfst
     bytes_to_skip(0), filename(std::string(filename)), has_hfst_header(false),
     hfst_version_2_weighted_transducer(false)
   {
-    try { 
-      if (strcmp("",filename.c_str()) != 0) {
-        std::ifstream ifs(filename.c_str());
-        if (ifs.fail())
-          HFST_THROW_MESSAGE(NotTransducerStreamException, 
-                             "file could not be opened");
-        input_stream = &ifs;
-        if (stream_eof())
-          HFST_THROW(EndOfStreamException);
-        type = stream_fst_type();
-      }
-      else {
-        input_stream = &std::cin;
-        if (stream_eof())
-          HFST_THROW(EndOfStreamException);
-        type = stream_fst_type();
-      }
+    if (strcmp("",filename.c_str()) != 0) {
+      std::ifstream ifs(filename.c_str());
+      if (ifs.fail())
+        HFST_THROW_MESSAGE(NotTransducerStreamException, 
+                           "file could not be opened");
+      input_stream = &ifs;
+      if (stream_eof())
+        HFST_THROW(EndOfStreamException);
+      type = stream_fst_type();
     }
-
-    catch( const HfstException e)
-      { throw e; }
+    else {
+      input_stream = &std::cin;
+      if (stream_eof())
+        HFST_THROW(EndOfStreamException);
+      type = stream_fst_type();
+    }
+    
     if ( ! HfstTransducer::is_implementation_type_available(type)) {
       HFST_THROW(ImplementationTypeNotAvailableException);
     }
-
+    
     switch (type)
-    {
+      {
 #if HAVE_SFST
-    case SFST_TYPE:
-      implementation.sfst 
-        = new hfst::implementations::SfstInputStream(filename);
-      break;
+      case SFST_TYPE:
+        implementation.sfst 
+          = new hfst::implementations::SfstInputStream(filename);
+        break;
 #endif
 #if HAVE_OPENFST
-    case TROPICAL_OPENFST_TYPE:
-      if (strcmp(filename.c_str(),"") == 0) {  
-        // FIXME: this should be done in TropicalWeight layer
-        implementation.tropical_ofst = 
-          new hfst::implementations::TropicalWeightInputStream();
-      }
-      else
-        implementation.tropical_ofst = 
-          new hfst::implementations::TropicalWeightInputStream(filename);
-      break;
+      case TROPICAL_OPENFST_TYPE:
+        if (strcmp(filename.c_str(),"") == 0) {  
+          // FIXME: this should be done in TropicalWeight layer
+          implementation.tropical_ofst = 
+            new hfst::implementations::TropicalWeightInputStream();
+        }
+        else
+          implementation.tropical_ofst = 
+            new hfst::implementations::TropicalWeightInputStream(filename);
+        break;
 #if HAVE_OPENFST_LOG
-    case LOG_OPENFST_TYPE:
-      implementation.log_ofst = 
-        new hfst::implementations::LogWeightInputStream(filename);
-      break;
+      case LOG_OPENFST_TYPE:
+        implementation.log_ofst = 
+          new hfst::implementations::LogWeightInputStream(filename);
+        break;
 #endif
 #endif
 #if HAVE_FOMA
-    case FOMA_TYPE:
-      implementation.foma 
-        = new hfst::implementations::FomaInputStream(filename);
-      break;
+      case FOMA_TYPE:
+        implementation.foma 
+          = new hfst::implementations::FomaInputStream(filename);
+        break;
 #endif
 #if HAVE_XFSM
-    case XFSM_TYPE:
-      implementation.xfsm 
-        = new hfst::implementations::XfsmInputStream(filename);
-      break;
+      case XFSM_TYPE:
+        implementation.xfsm 
+          = new hfst::implementations::XfsmInputStream(filename);
+        break;
 #endif
 #if HAVE_MY_TRANSDUCER_LIBRARY
-    case MY_TRANSDUCER_LIBRARY_TYPE:
-      implementation.my_transducer_library 
-        = new hfst::implementations::MyTransducerLibraryInputStream(filename);
-      break;
+      case MY_TRANSDUCER_LIBRARY_TYPE:
+        implementation.my_transducer_library 
+          = new hfst::implementations::MyTransducerLibraryInputStream(filename);
+        break;
 #endif
-    case HFST_OL_TYPE:
-      implementation.hfst_ol 
-        = new hfst::implementations::HfstOlInputStream(filename, false);
-      break;
-    case HFST_OLW_TYPE:
-      implementation.hfst_ol 
-        = new hfst::implementations::HfstOlInputStream(filename, true);
-      break;
-    default:
-      debug_error("#10");
-
-      HFST_THROW_MESSAGE(NotTransducerStreamException,
-                         "transducer type not recognised");
-    }
+      case HFST_OL_TYPE:
+        implementation.hfst_ol 
+          = new hfst::implementations::HfstOlInputStream(filename, false);
+        break;
+      case HFST_OLW_TYPE:
+        implementation.hfst_ol 
+          = new hfst::implementations::HfstOlInputStream(filename, true);
+        break;
+      default:
+        debug_error("#10");
+        
+        HFST_THROW_MESSAGE(NotTransducerStreamException,
+                           "transducer type not recognised");
+      }
   }
-
+  
   HfstInputStream::~HfstInputStream(void)
   { 
     switch (type)
@@ -1309,6 +1313,12 @@ namespace hfst
   {
     return type;
   }
+
+  bool HfstInputStream::is_hfst_header_included(void) const
+  {
+    return has_hfst_header;
+  }
+
 }
 
 #else // MAIN_TEST was defined
