@@ -1,14 +1,12 @@
-//       This program is free software: you can redistribute it and/or modify
-//       it under the terms of the GNU General Public License as published by
-//       the Free Software Foundation, version 3 of the License.
-//
-//       This program is distributed in the hope that it will be useful,
-//       but WITHOUT ANY WARRANTY; without even the implied warranty of
-//       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//       GNU General Public License for more details.
-//
-//       You should have received a copy of the GNU General Public License
-//       along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// Copyright (c) 2016 University of Helsinki                          
+//                                                                    
+// This library is free software; you can redistribute it and/or      
+// modify it under the terms of the GNU Lesser General Public         
+// License as published by the Free Software Foundation; either       
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more      
+// information.
+
 #ifndef _HFST_INPUTSTREAM_H_
 #define _HFST_INPUTSTREAM_H_
 
@@ -193,8 +191,9 @@ For documentation on the HFST binary transducer format, see
       OPENFST_LOG_,
       /* An SFST transducer. */ 
       SFST_, 
-      /* A foma transducer. */ 
-      FOMA_, 
+      /* A foma transducer in unzipped format. 
+         A zipped file is handled by throwing a FileIsInGZFormatException. */ 
+      FOMA_,
       /* An xfsm transducer. */
       XFSM_,
       /* Your transducer type */
@@ -270,6 +269,8 @@ For documentation on the HFST binary transducer format, see
     transducer that has a different type than the previous ones. */
     HFSTDLL ImplementationType get_type(void) const;
 
+    HFSTDLL bool is_hfst_header_included(void) const;
+
     friend class HfstTransducer;
   };
 
