@@ -1,14 +1,11 @@
-//       This program is free software: you can redistribute it and/or modify
-//       it under the terms of the GNU General Public License as published by
-//       the Free Software Foundation, version 3 of the License.
-//
-//       This program is distributed in the hope that it will be useful,
-//       but WITHOUT ANY WARRANTY; without even the implied warranty of
-//       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//       GNU General Public License for more details.
-//
-//       You should have received a copy of the GNU General Public License
-//       along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// Copyright (c) 2016 University of Helsinki                          
+//                                                                    
+// This library is free software; you can redistribute it and/or      
+// modify it under the terms of the GNU Lesser General Public         
+// License as published by the Free Software Foundation; either       
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more      
+// information.
 
 #include "HfstTokenizer.h"
 #include "HfstFlagDiacritics.h"
@@ -112,7 +109,7 @@ const
 { return (s == "") || (skip_symbol_set.find(s) != skip_symbol_set.end()); }
 
 void
-HfstTokenizer::add_multichar_symbol(const string& symbol)
+HfstTokenizer::add_multichar_symbol(const std::string& symbol)
 {  if (symbol == "")
     { return; }
   multi_char_symbols.add(symbol.c_str()); }
@@ -125,7 +122,7 @@ HfstTokenizer::add_skip_symbol(const std::string &symbol)
   skip_symbol_set.insert(symbol.c_str()); }
 
 StringPairVector HfstTokenizer::tokenize
-(const string& input_string) const
+(const std::string& input_string) const
 {
   check_utf8_correctness(input_string);
   StringPairVector spv;
@@ -143,7 +140,7 @@ StringPairVector HfstTokenizer::tokenize
 }
 
 StringVector HfstTokenizer::tokenize_one_level
-(const string& input_string) const
+(const std::string& input_string) const
 {
   check_utf8_correctness(input_string);
 
@@ -200,7 +197,7 @@ StringPairVector HfstTokenizer::tokenize_space_separated(const std::string & str
 }
 
 StringPairVector HfstTokenizer::tokenize
-(const string& input_string,const string& output_string) const
+(const std::string& input_string,const std::string& output_string) const
 {
   check_utf8_correctness(input_string);
   check_utf8_correctness(output_string);
@@ -238,7 +235,7 @@ StringPairVector HfstTokenizer::tokenize
 }
 
 StringPairVector HfstTokenizer::tokenize
-(const string& input_string,const string& output_string,
+(const std::string& input_string,const std::string& output_string,
  void (*warn_about_pair)(const std::pair<std::string, std::string> &symbol_pair)) const
 {
   check_utf8_correctness(input_string);
@@ -284,7 +281,7 @@ StringPairVector HfstTokenizer::tokenize
 }
 
 StringPairVector HfstTokenizer::tokenize_and_align_flag_diacritics
-(const string& input_string,const string& output_string,
+(const std::string& input_string,const std::string& output_string,
  void (*warn_about_pair)(const std::pair<std::string, std::string> &symbol_pair)) const
 {
   check_utf8_correctness(input_string);
