@@ -1,14 +1,12 @@
-//       This program is free software: you can redistribute it and/or modify
-//       it under the terms of the GNU General Publirmoc License as published by
-//       the Free Software Foundation, version 3 of the License.
+// Copyright (c) 2016 University of Helsinki
 //
-//       This program is distributed in the hope that it will be useful,
-//       but WITHOUT ANY WARRANTY; without even the implied warranty of
-//       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//       GNU General Public License for more details.
-//
-//       You should have received a copy of the GNU General Public License
-//       along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// This library is free software; you can redistribute it and/or
+// modify it under the terms of the GNU Lesser General Public
+// License as published by the Free Software Foundation; either
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more
+// information.
+
 #ifndef _HFST_TRANSDUCER_H_
 #define _HFST_TRANSDUCER_H_
 
@@ -534,6 +532,8 @@ An example:
        by \a type. */
     HFSTDLL HfstTransducer(const StringPairVector & spv, ImplementationType type);
 
+    HFSTDLL HfstTransducer(const StringVector & sv, ImplementationType type);
+
     /* \brief Create a transducer that recognizes the concatenation of the 
        unions of string pairs in string pair sets in \a spsv. The type of
        the transducer is defined by \a type. */
@@ -1083,6 +1083,7 @@ ccc : ddd
     //! @param limit  (Currently ignored.) Number of strings to look up. 
     //!               -1 tries to look up all and may get stuck 
     //!               if infinitely ambiguous.
+    //! @param time_cutoff Number of seconds that can pass before lookup is stopped.
     //! \return{A pointer to a HfstOneLevelPaths container allocated by callee}
     //! 
     //! @see HfstTokenizer::tokenize_one_level
@@ -1108,6 +1109,7 @@ ccc : ddd
     //! @param limit  (Currently ignored.) Number of strings to look up. 
     //!               -1 tries to look up all and may get stuck 
     //!               if infinitely ambiguous.
+    //! @param time_cutoff Number of seconds that can pass before lookup is stopped.
     //! \return{A pointer to a HfstOneLevelPaths container allocated by callee}
     //!
     //!
@@ -1121,7 +1123,7 @@ ccc : ddd
     //!
     //!
     //! This function is the same as 
-    //! #lookup(const StringVector&, ssize_t) const
+    //! #lookup(const StringVector&, ssize_t, double) const
     //! but lookup is not done using a string and a tokenizer instead of
     //! a StringVector.
       HFSTDLL HfstOneLevelPaths * lookup(const HfstTokenizer& tok,
@@ -1134,7 +1136,7 @@ ccc : ddd
     //! \a tok defines how s is tokenized.
     //!
     //! The same as 
-    //! #lookup_fd(const StringVector&, ssize_t) const 
+    //! #lookup_fd(const StringVector&, ssize_t, double) const 
     //! but uses a tokenizer and a string instead of a StringVector.
     //!
     HFSTDLL HfstOneLevelPaths * lookup_fd(
