@@ -1,14 +1,11 @@
-//       This program is free software: you can redistribute it and/or modify
-//       it under the terms of the GNU General Public License as published by
-//       the Free Software Foundation, version 3 of the License.
-//
-//       This program is distributed in the hope that it will be useful,
-//       but WITHOUT ANY WARRANTY; without even the implied warranty of
-//       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//       GNU General Public License for more details.
-//
-//       You should have received a copy of the GNU General Public License
-//       along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// Copyright (c) 2016 University of Helsinki                          
+//                                                                    
+// This library is free software; you can redistribute it and/or      
+// modify it under the terms of the GNU Lesser General Public         
+// License as published by the Free Software Foundation; either       
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more      
+// information.
 
 #if HAVE_CONFIG_H
 #  include <config.h>
@@ -459,12 +456,19 @@ HfstTransducer * ConversionFunctions::hfst_ol_to_hfst_transducer(
 {
     hfst::ImplementationType type = t->is_weighted() ? HFST_OLW_TYPE : HFST_OL_TYPE;
     HfstTransducer * retval = new HfstTransducer(type);
+    delete retval->implementation.hfst_ol;
     retval->implementation.hfst_ol = new hfst_ol::Transducer(*t);
     return retval;
 }
 
-
-
+hfst_ol::Transducer * ConversionFunctions::hfst_transducer_to_hfst_ol(
+    HfstTransducer * t)
+{
+    if (t->type != HFST_OL_TYPE && t->type != HFST_OLW_TYPE) {
+        t->convert(HFST_OLW_TYPE);
+    }
+    return t->implementation.hfst_ol;
+}
 
 }}
 #else // MAIN_TEST was defined
