@@ -1,14 +1,11 @@
-//       This program is free software: you can redistribute it and/or modify
-//       it under the terms of the GNU General Public License as published by
-//       the Free Software Foundation, version 3 of the License.
-//
-//       This program is distributed in the hope that it will be useful,
-//       but WITHOUT ANY WARRANTY; without even the implied warranty of
-//       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//       GNU General Public License for more details.
-//
-//       You should have received a copy of the GNU General Public License
-//       along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// Copyright (c) 2016 University of Helsinki                          
+//                                                                    
+// This library is free software; you can redistribute it and/or      
+// modify it under the terms of the GNU Lesser General Public         
+// License as published by the Free Software Foundation; either       
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more      
+// information.
 
 #ifndef _CONVERT_TRANSDUCER_H_
 #define _CONVERT_TRANSDUCER_H_
@@ -224,7 +221,6 @@ namespace implementations {
 #endif // HAVE_OPENFST 
   
   
-  
   static HfstBasicTransducer * hfst_ol_to_hfst_basic_transducer
     (hfst_ol::Transducer * t);
 
@@ -233,9 +229,10 @@ namespace implementations {
        std::string options="", HfstTransducer * harmonizer = NULL);
 
   // A way to smuggle a hfst_ol backend into a HfstTransducer wrapper
-  static HfstTransducer * hfst_ol_to_hfst_transducer
-      (hfst_ol::Transducer * t);
+  static HfstTransducer * hfst_ol_to_hfst_transducer(hfst_ol::Transducer * t);
 
+  // And the reverse
+  static hfst_ol::Transducer * hfst_transducer_to_hfst_ol(HfstTransducer * t);
 
   /* Define here the functions that convert between HfstBasicTransducer and 
      your transducer class. */
