@@ -1,3 +1,12 @@
+// Copyright (c) 2016 University of Helsinki                          
+//                                                                    
+// This library is free software; you can redistribute it and/or      
+// modify it under the terms of the GNU Lesser General Public         
+// License as published by the Free Software Foundation; either       
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more      
+// information.
+
 #ifndef _HFST_TRANSITION_H_
 #define _HFST_TRANSITION_H_
 
@@ -136,12 +145,7 @@ namespace hfst {
     typedef HfstTransition<HfstTropicalTransducerTransitionData> 
       HfstBasicTransition;
 
-    /** @brief An HfstTransition with transition data of type
-        HfstFastTransitionData. 
-
-        This implementation is compatible with #HfstFastTransducer.
-
-        @see HfstFastTransitionData HfstFastTransducer */
+    // TODO: remove?
     typedef HfstTransition<HfstFastTransitionData> HfstFastTransition;
 
   }
