@@ -1,14 +1,12 @@
-//       This program is free software: you can redistribute it and/or modify
-//       it under the terms of the GNU General Public License as published by
-//       the Free Software Foundation, version 3 of the License.
-//
-//       This program is distributed in the hope that it will be useful,
-//       but WITHOUT ANY WARRANTY; without even the implied warranty of
-//       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//       GNU General Public License for more details.
-//
-//       You should have received a copy of the GNU General Public License
-//       along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// Copyright (c) 2016 University of Helsinki                          
+//                                                                    
+// This library is free software; you can redistribute it and/or      
+// modify it under the terms of the GNU Lesser General Public         
+// License as published by the Free Software Foundation; either       
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more      
+// information.
+
 #ifndef _TROPICAL_WEIGHT_TRANSDUCER_H_
 #define _TROPICAL_WEIGHT_TRANSDUCER_H_
 
@@ -211,8 +209,8 @@ namespace implementations
       static StdVectorFst * disjunct
         (StdVectorFst * t, const NumberPairVector &npv);
 
-      static StdVectorFst * disjunct_as_tries(StdVectorFst * t1,
-                                              const StdVectorFst * t2);
+      static fst::StdVectorFst * disjunct_as_tries(fst::StdVectorFst * t1,
+                                              const fst::StdVectorFst * t2);
 
       static StdVectorFst * intersect(StdVectorFst * t1,
                                       StdVectorFst * t2);
@@ -333,9 +331,9 @@ namespace implementations
                   StdArc::StateId sourcestate,                          
                   StdArc::Label ilabel, 
                   StdArc::Label olabel);
-      static void disjunct_as_tries(StdVectorFst &t1,
+      static void disjunct_as_tries(fst::StdVectorFst &t1,
                              StateId t1_state,
-                             const StdVectorFst * t2,
+                                    const fst::StdVectorFst * t2,
                              StateId t2_state);
       static void add_sub_trie(StdVectorFst &t1,
                         StateId t1_state,
