@@ -1,3 +1,11 @@
+// Copyright (c) 2016 University of Helsinki                          
+//                                                                    
+// This library is free software; you can redistribute it and/or      
+// modify it under the terms of the GNU Lesser General Public         
+// License as published by the Free Software Foundation; either       
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more      
+// information.
 #include "pmatch.h"
 #include "hfst.h"
 
@@ -6,7 +14,7 @@ namespace hfst_ol {
 PmatchAlphabet::PmatchAlphabet(std::istream & inputstream,
                                SymbolNumber symbol_count):
     TransducerAlphabet(inputstream, symbol_count, false),
-    special_symbols(12, NO_SYMBOL_NUMBER) // SpecialSymbols enum
+    special_symbols(SPECIALSYMBOL_NR_ITEMS, NO_SYMBOL_NUMBER) // SpecialSymbols enum
 {
     symbol2lists = SymbolNumberVector(orig_symbol_count, NO_SYMBOL_NUMBER);
     list2symbols = SymbolNumberVector(orig_symbol_count, NO_SYMBOL_NUMBER);
@@ -22,17 +30,49 @@ PmatchAlphabet::PmatchAlphabet(std::istream & inputstream,
     }
 }
 
+PmatchAlphabet::PmatchAlphabet(TransducerAlphabet const & a):
+    TransducerAlphabet(a),
+    special_symbols(SPECIALSYMBOL_NR_ITEMS, NO_SYMBOL_NUMBER) {
+    symbol2lists = SymbolNumberVector(orig_symbol_count, NO_SYMBOL_NUMBER);
+    list2symbols = SymbolNumberVector(orig_symbol_count, NO_SYMBOL_NUMBER);
+    rtns = RtnVector(orig_symbol_count, NULL);
+    // We initialize the vector of which symbols have a printable representation
+    // with false, then flip those that actually do to true
+    printable_vector = std::vector<bool>(orig_symbol_count, false);
+    for (SymbolNumber i = 1; i < symbol_table.size(); ++i) {
+        add_special_symbol(symbol_table[i], i);
+        if (is_flag_diacritic(i)) {
+            printable_vector[i] = false;
+        }
+    }
+}
+
 PmatchAlphabet::PmatchAlphabet(void):
     TransducerAlphabet()
 {}
 
 void PmatchAlphabet::add_symbol(const std::string & symbol)
 {
-    TransducerAlphabet::add_symbol(symbol);
     symbol2lists.push_back(NO_SYMBOL_NUMBER);
     list2symbols.push_back(NO_SYMBOL_NUMBER);
     rtns.push_back(NULL);
     printable_vector.push_back(true);
+    if (exclusionary_lists.size() != 0) {
+        // if there are exclusionary lists, they should all accept the new symbol
+        symbol2lists[symbol_table.size()] = symbol_lists.size();
+        symbol_lists.push_back(SymbolNumberVector(exclusionary_lists.begin(),
+                                                  exclusionary_lists.end()));
+#ifndef _MSC_VER
+        for(const auto & exc: exclusionary_lists) {
+          symbol_list_members[list2symbols[exc]].push_back(symbol_table.size());
+        }
+#else
+        for (SymbolNumberVector::const_iterator exc = exclusionary_lists.begin(); exc != exclusionary_lists.end(); exc++) {
+          symbol_list_members[list2symbols[*exc]].push_back(symbol_table.size());
+        }
+#endif
+    }
+    TransducerAlphabet::add_symbol(symbol);
 }
 
 bool PmatchAlphabet::is_printable(SymbolNumber symbol)
@@ -65,6 +105,8 @@ void PmatchAlphabet::add_special_symbol(const std::string & str,
         special_symbols[NRC_exit] = symbol_number;
     } else if (str == "@PMATCH_PASSTHROUGH@") {
         special_symbols[Pmatch_passthrough] = symbol_number;
+    } else if (str == "@PMATCH_INPUT_MARK@") {
+        special_symbols[Pmatch_input_mark] = symbol_number;
     } else if (str == "@BOUNDARY@") {
         special_symbols[boundary] = symbol_number;
     } else if (is_end_tag(str)) {
@@ -81,6 +123,7 @@ void PmatchAlphabet::add_special_symbol(const std::string & str,
     } else if (is_counter(str)) {
         process_counter(str, symbol_number);
     } else {
+        // it's a regular symbol
         printable_vector[symbol_number] = true;
     }
 }
@@ -89,7 +132,9 @@ void PmatchAlphabet::process_symbol_list(std::string str, SymbolNumber sym)
 {
     SymbolNumberVector list_symbols;
     StringSymbolMap ss = build_string_symbol_map();
-    size_t begin = strlen("@PMATCH_LIST_");
+    // regular list or exlusionary list?
+    bool polarity = str[1] == 'L';
+    size_t begin = strlen("@L.");
     size_t stop;
     std::vector<std::string> collected_symbols;
     while ((stop = str.find('_', begin)) != std::string::npos) {
@@ -104,8 +149,6 @@ void PmatchAlphabet::process_symbol_list(std::string str, SymbolNumber sym)
         }
         collected_symbols.push_back(symbol);
     }
-    // One at the end
-    collected_symbols.push_back(str.substr(begin, str.size() - begin - strlen("@")));
     // Process the symbols we found
     for (std::vector<std::string>::const_iterator it = collected_symbols.begin();
          it != collected_symbols.end(); ++it) {
@@ -119,15 +162,35 @@ void PmatchAlphabet::process_symbol_list(std::string str, SymbolNumber sym)
             str_sym = ss[*it];
         }
         list_symbols.push_back(str_sym);
-        if (symbol2lists[str_sym] == NO_SYMBOL_NUMBER) {
-            symbol2lists[str_sym] = symbol_lists.size();
-            symbol_lists.push_back(SymbolNumberVector(1, sym));
-        } else {
-            symbol_lists[symbol2lists[str_sym]].push_back(sym);
+        if (polarity == true) {
+            if (symbol2lists[str_sym] == NO_SYMBOL_NUMBER) {
+                symbol2lists[str_sym] = symbol_lists.size();
+                symbol_lists.push_back(SymbolNumberVector(1, sym));
+            } else {
+                symbol_lists[symbol2lists[str_sym]].push_back(sym);
+            }
         }
     }
     list2symbols[sym] = symbol_list_members.size();
-    symbol_list_members.push_back(list_symbols);
+    if (polarity == false) {
+        SymbolNumberVector excl_symbols;
+        exclusionary_lists.push_back(sym);
+        for (SymbolNumber candidate_for_list = 1; candidate_for_list < symbol_table.size(); ++candidate_for_list) {
+            if (is_printable(symbol_table[candidate_for_list]) &&
+                find(list_symbols.begin(), list_symbols.end(), candidate_for_list) == list_symbols.end()) {
+                excl_symbols.push_back(candidate_for_list);
+                if (symbol2lists[candidate_for_list] == NO_SYMBOL_NUMBER) {
+                    symbol2lists[candidate_for_list] = symbol_lists.size();
+                    symbol_lists.push_back(SymbolNumberVector(1, sym));
+                } else {
+                    symbol_lists[symbol2lists[sym]].push_back(sym);
+                }
+            }
+        }
+        symbol_list_members.push_back(excl_symbols);
+    } else {
+        symbol_list_members.push_back(list_symbols);
+    }
 }
 
 SymbolNumberVector PmatchAlphabet::get_specials(void) const
@@ -263,7 +326,104 @@ PmatchContainer::PmatchContainer(std::istream & inputstream):
             }
         }
     }
+    
+}
 
+PmatchContainer::PmatchContainer(Transducer * t):
+    verbose(false),
+    locate_mode(false),
+    profile_mode(false),
+    single_codepoint_tokenization(false),
+    recursion_depth_left(PMATCH_MAX_RECURSION_DEPTH),
+    entry_stack()
+{
+    TransducerHeader header = t->get_header();
+    alphabet = PmatchAlphabet(t->get_alphabet());
+    orig_symbol_count = symbol_count = alphabet.get_orig_symbol_count();
+    alphabet.extract_tags = locate_mode;
+    line_number = 0;
+    encoder = new Encoder(alphabet.get_symbol_table(), orig_symbol_count);
+    TransducerTable<TransitionW> transitions = t->copy_transitionw_table();
+    TransducerTable<TransitionWIndex> indices = t->copy_windex_table();
+    toplevel = new hfst_ol::PmatchTransducer(
+        transitions.get_vector(),
+        indices.get_vector(),
+        alphabet,
+        this);
+    toplevel->collect_possible_first_symbols();
+
+    // Finally fetch the first symbols from any
+    // first-position rtn arcs in TOP. If they are potential epsilon loops,
+    // clear out the set.
+    SymbolNumber max_input_sym = 0;
+    std::set<SymbolNumber> & possible_firsts = toplevel->possible_first_symbols;
+    for (std::set<SymbolNumber>::iterator it = possible_firsts.begin();
+         it != possible_firsts.end(); ++it) {
+        if (*it > max_input_sym) { max_input_sym = *it; }
+        if (alphabet.has_rtn(*it)) {
+            if (alphabet.get_rtn(*it) == toplevel) {
+                possible_firsts.clear();
+                break;
+            }
+            alphabet.get_rtn(*it)->collect_possible_first_symbols();
+            std::set<SymbolNumber> rtn_firsts =
+                alphabet.get_rtn(*it)->possible_first_symbols;
+            for (RtnNameMap::const_iterator it = alphabet.rtn_names.begin();
+                 it != alphabet.rtn_names.end(); ++it) {
+                if (rtn_firsts.count(it->second) == 1) {
+                    // For now we are very conservative:
+                    // if we can go through two levels of rtns
+                    // without any input, we just assume the full
+                    // input set is possible
+                    possible_firsts.clear();
+                }
+            }
+            if (rtn_firsts.empty() || possible_firsts.empty()) {
+                possible_firsts.clear();
+                break;
+            } else {
+                for (std::set<SymbolNumber>::
+                         const_iterator rtn_it = rtn_firsts.begin();
+                     rtn_it != rtn_firsts.end(); ++rtn_it) {
+                    if (*rtn_it > max_input_sym) { max_input_sym = *rtn_it ;}
+                    possible_firsts.insert(*rtn_it);
+                }
+            }
+        }
+    }
+    for (RtnNameMap::const_iterator it = alphabet.rtn_names.begin();
+         it != alphabet.rtn_names.end(); ++it) {
+        possible_firsts.erase(it->second);
+    }
+    if (!possible_firsts.empty() &&
+        alphabet.get_special(boundary) != NO_SYMBOL_NUMBER) {
+        possible_firsts.insert(alphabet.get_special(boundary));
+    }
+    if (!possible_firsts.empty()) {
+        for (int i = 0; i <= max_input_sym; ++i) {
+            if (possible_firsts.count(i) == 1) {
+                possible_first_symbols.push_back(1);
+            } else {
+                possible_first_symbols.push_back(0);
+            }
+        }
+    }
+}
+
+void PmatchContainer::add_rtn(Transducer * rtn, const std::string & name)
+{
+    TransducerTable<TransitionW> transitions = rtn->copy_transitionw_table();
+    TransducerTable<TransitionWIndex> indices = rtn->copy_windex_table();
+    PmatchTransducer * pmatch_rtn = new hfst_ol::PmatchTransducer(
+        transitions.get_vector(),
+        indices.get_vector(),
+        alphabet,
+        this);
+    if (!alphabet.has_rtn(name)) {
+        alphabet.add_rtn(pmatch_rtn, name);
+    } else {
+        delete rtn;
+    }
 }
 
 PmatchContainer::PmatchContainer(void)
@@ -299,7 +459,7 @@ bool PmatchAlphabet::is_counter(const std::string & symbol)
 
 bool PmatchAlphabet::is_list(const std::string & symbol)
 {
-    return symbol.find("@PMATCH_LIST_") == 0 && symbol.rfind("@") == symbol.size() - 1;
+    return (symbol.find("@L.") == 0 || symbol.find("@X.") == 0) && symbol.rfind("@") == symbol.size() - 1;
 }
 
 bool PmatchAlphabet::is_special(const std::string & symbol)
@@ -315,6 +475,14 @@ bool PmatchAlphabet::is_special(const std::string & symbol)
     }
 }
 
+bool PmatchAlphabet::is_printable(const std::string & symbol)
+{
+    if (symbol.size() < 3) {
+        return true;
+    }
+    return symbol.find("@") != 0 || symbol.at(symbol.size() - 1) != '@';
+}
+
 bool PmatchAlphabet::is_guard(const SymbolNumber symbol) const
 {
     for(SymbolNumberVector::const_iterator it = guards.begin();
@@ -331,6 +499,11 @@ bool PmatchAlphabet::is_counter(const SymbolNumber symbol) const
     return (symbol < counters.size() && counters[symbol] != NO_COUNTER);
 }
 
+bool PmatchAlphabet::is_input_mark(const SymbolNumber symbol) const
+{
+    return get_special(Pmatch_input_mark) == symbol;
+}
+
 std::string PmatchAlphabet::name_from_insertion(const std::string & symbol)
 {
     return symbol.substr(sizeof("@I.") - 1, symbol.size() - (sizeof("@I.@") - 1));
@@ -477,7 +650,7 @@ SymbolNumber PmatchAlphabet::get_special(SpecialSymbol special) const
     return special_symbols.at(special);
 }
 
-void PmatchContainer::process(std::string & input_str)
+void PmatchContainer::process(const std::string & input_str)
 {
     initialize_input(input_str.c_str());
     unsigned int input_pos = 0;
@@ -548,15 +721,29 @@ void PmatchContainer::process(std::string & input_str)
     }
 }
 
-std::string PmatchContainer::match(std::string & input)
+std::string PmatchContainer::match(const std::string & input,
+                                   double time_cutoff)
 {
+    max_time = time_cutoff;
+    if (max_time > 0.0) {
+        start_clock = clock();
+        call_counter = 0;
+        limit_reached = false;
+    }
     locate_mode = false;
     process(input);
     return stringify_output();
 }
 
-LocationVectorVector PmatchContainer::locate(std::string & input)
+LocationVectorVector PmatchContainer::locate(std::string & input,
+                                             double time_cutoff)
 {
+    max_time = time_cutoff;
+    if (max_time > 0.0) {
+        start_clock = clock();
+        call_counter = 0;
+        limit_reached = false;
+    }
     locate_mode = true;
     process(input);
     return locations;
@@ -665,10 +852,11 @@ Location PmatchAlphabet::locatefy(unsigned int input_offset,
     Location retval;
     retval.start = input_offset;
     retval.weight = str.weight;
+    size_t input_mark = 0;
+    size_t output_mark = 0;
 
     // We rebuild the original input without special
     // symbols but with IDENTITIES etc. replaced
-    SymbolNumberVector orig_input;
     for (DoubleTape::const_iterator it = str.begin();
          it != str.end(); ++it) {
         SymbolNumber input = it->input;
@@ -681,15 +869,23 @@ Location PmatchAlphabet::locatefy(unsigned int input_offset,
             retval.output.append(string_from_symbol(output));
         }
         if (is_printable(input)) {
-            orig_input.push_back(input);
+            retval.input.append(string_from_symbol(input));
             ++input_offset;
         }
+        if (is_input_mark(output)) {
+            retval.output_parts.push_back(output_mark);
+            retval.input_parts.push_back(input_mark);
+            output_mark = retval.output.size();
+            input_mark = retval.input.size();
+        }
+    }
+    if (output_mark > 0) {
+        retval.output_parts.push_back(output_mark);
+    }
+    if (input_mark > 0) {
+        retval.input_parts.push_back(input_mark);
     }
     retval.length = input_offset - retval.start;
-    for(SymbolNumberVector::const_iterator input_it = orig_input.begin();
-        input_it != orig_input.end(); ++input_it) {
-        retval.input.append(string_from_symbol(*input_it));
-    }
     return retval;
 }
 
@@ -767,6 +963,36 @@ PmatchTransducer::PmatchTransducer(std::istream & is,
     free(orig_p);
 }
 
+PmatchTransducer::PmatchTransducer(std::vector<TransitionW> transition_vector,
+                                   std::vector<TransitionWIndex> index_vector,
+                                   PmatchAlphabet & alpha,
+                                   PmatchContainer * cont):
+    alphabet(alpha),
+    container(cont),
+    locations(NULL),
+    transition_table(transition_vector),
+    index_table(index_vector)
+{
+    orig_symbol_count = alphabet.get_symbol_table().size();
+    // initialize the stack for local variables
+    LocalVariables locals_front;
+    locals_front.flag_state = alphabet.get_fd_table();
+    locals_front.tape_step = 1;
+    locals_front.context = none;
+    locals_front.context_placeholder = 0;
+    locals_front.default_symbol_trap = false;
+    locals_front.negative_context_success = false;
+    locals_front.pending_passthrough = false;
+    locals_front.running_weight = 0.0;
+    local_stack.push(locals_front);
+    RtnVariables rtn_front;
+    rtn_front.tape_entry = 0;
+    rtn_front.input_tape_entry = 0;
+    rtn_front.candidate_input_pos = 0;
+    rtn_front.candidate_tape_pos = 0;
+    rtn_stack.push(rtn_front);
+}
+
 // Precompute which symbols may be at the start of a match.
 // For now we ignore default arcs, as does the rest of pmatch.
 void PmatchTransducer::collect_possible_first_symbols(void)
@@ -878,7 +1104,15 @@ void PmatchTransducer::collect_first_transition(TransitionTableIndex i,
                     throw true;
                 }
                 if (alphabet.list2symbols[*it] != NO_SYMBOL_NUMBER) {
-// If this is a list, collect everything in the list
+                    // If this is a list, collect everything in the list
+                    // if it's an exclusionary list, give up
+                    if (std::find(alphabet.exclusionary_lists.begin(),
+                                  alphabet.exclusionary_lists.end(),
+                                  *it)
+                        != alphabet.exclusionary_lists.end()) {
+                        container->reset_recursion();
+                        throw true;
+                    }
                     for (SymbolNumberVector::const_iterator sym_it =
                             alphabet.symbol_list_members[alphabet.list2symbols[*it]].begin();
                         sym_it != alphabet.symbol_list_members[alphabet.list2symbols[*it]].end(); ++sym_it) {
@@ -1021,6 +1255,7 @@ void PmatchTransducer::match(unsigned int & input_tape_pos,
     rtn_stack.top().tape_entry = tape_pos;
     rtn_stack.top().candidate_tape_pos = tape_pos;
     rtn_stack.top().best_weight = 0.0;
+    rtn_stack.top().candidate_found = false;
     local_stack.top().context = none;
     local_stack.top().tape_step = 1;
     local_stack.top().context_placeholder = 0;
@@ -1047,6 +1282,7 @@ void PmatchTransducer::rtn_call(unsigned int & input_tape_pos,
     rtn_stack.top().tape_entry = tape_pos;
     rtn_stack.top().candidate_tape_pos = tape_pos;
     rtn_stack.top().best_weight = 0.0;
+    rtn_stack.top().candidate_found = false;
     local_stack.push(local_stack.top());
     local_stack.top().flag_state = alphabet.get_fd_table();
     local_stack.top().tape_step = 1;
@@ -1073,7 +1309,7 @@ void PmatchTransducer::note_analysis(unsigned int input_pos,
         // with left contexts and should be dealt with a bit more nicely
         return;
     }
-
+    rtn_stack.top().candidate_found = true;
     if (locations != NULL) {
         grab_location(input_pos, tape_pos);
         return;
@@ -1191,7 +1427,6 @@ void PmatchTransducer::check_context(unsigned int input_pos,
                                      unsigned int tape_pos,
                                      TransitionTableIndex i)
 {
-//    std::cerr << local_stack.size() << std::endl;
     local_stack.top().context_placeholder = input_pos;
     if (local_stack.top().context == LC ||
         local_stack.top().context == NLC) {
@@ -1214,7 +1449,6 @@ void PmatchTransducer::check_context(unsigned int input_pos,
     if (schedule_passthrough) {
         local_stack.top().pending_passthrough = true;
     }
-//    std::cerr << local_stack.size() << std::endl;
 }
 
 void PmatchTransducer::take_rtn(SymbolNumber input,
@@ -1320,6 +1554,18 @@ void PmatchTransducer::get_analyses(unsigned int input_pos,
                                     unsigned int tape_pos,
                                     TransitionTableIndex i)
 {
+    if (container->max_time > 0.0) {
+        ++container->call_counter;
+        // Have we spent too much time?
+        if (container->limit_reached ||
+            (container->call_counter % 1000000 == 0 &&
+             (rtn_stack.top().candidate_found &&
+              // if we have at least something, stop doing more work
+              (((double)(clock() - container->start_clock)) / CLOCKS_PER_SEC) > container->max_time))) {
+            container->limit_reached = true;
+            return;
+        }
+    }
     if (!container->try_recurse()) {
         if (container->verbose) {
             std::cerr << "pmatch: out of stack space, truncating result\n";
@@ -1352,14 +1598,14 @@ void PmatchTransducer::get_analyses(unsigned int input_pos,
     }
     
     if (alphabet.symbol2lists[input] != NO_SYMBOL_NUMBER) {
-// At least one symbol list contains this symbol
+// At least one symbol list could allow this symbol
         for(SymbolNumberVector::const_iterator it =
                 alphabet.symbol_lists[alphabet.symbol2lists[input]].begin();
             it != alphabet.symbol_lists[alphabet.symbol2lists[input]].end(); ++it) {
             take_transitions(*it, input_pos, tape_pos, i+1);
         }
     }
-
+    // The "normal" case where we have a regular input symbol
     if (input < orig_symbol_count) {
         take_transitions(input, input_pos, tape_pos, i+1);
     } else {
@@ -1370,6 +1616,7 @@ void PmatchTransducer::get_analyses(unsigned int input_pos,
             take_transitions(alphabet.get_unknown_symbol(), input_pos, tape_pos, i+1);
         }
     }
+    
     container->unrecurse();
 }
 
