@@ -1,14 +1,11 @@
-//       This program is free software: you can redistribute it and/or modify
-//       it under the terms of the GNU General Public License as published by
-//       the Free Software Foundation, version 3 of the License.
-//
-//       This program is distributed in the hope that it will be useful,
-//       but WITHOUT ANY WARRANTY; without even the implied warranty of
-//       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//       GNU General Public License for more details.
-//
-//       You should have received a copy of the GNU General Public License
-//       along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// Copyright (c) 2016 University of Helsinki                          
+//                                                                    
+// This library is free software; you can redistribute it and/or      
+// modify it under the terms of the GNU Lesser General Public         
+// License as published by the Free Software Foundation; either       
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more      
+// information.
 
 #include "./transducer.h"
 
@@ -283,6 +280,20 @@ bool Transducer::initialize_input(const char * input)
     return true;
 }
 
+void Transducer::include_symbol_in_alphabet(const std::string & sym)
+{
+    SymbolNumber key = alphabet->symbol_from_string(sym);
+    if (key != NO_SYMBOL_NUMBER) {
+        return;
+    }
+    key = alphabet->get_symbol_table().size();
+    alphabet->add_symbol(sym);
+    char * cstr_for_encoder = new char[sym.size() + 1];
+    std::strcpy(cstr_for_encoder, sym.c_str());
+    encoder->read_input_symbol(cstr_for_encoder, key);
+    delete[] cstr_for_encoder;
+}
+
 HfstOneLevelPaths * Transducer::lookup_fd(const StringVector & s, ssize_t limit,
                                           double time_cutoff)
 {
@@ -332,7 +343,7 @@ HfstOneLevelPaths * Transducer::lookup_fd(const char * s, ssize_t limit,
     max_time = 0.0;
     if (time_cutoff > 0.0) {
         max_time = time_cutoff;
-        max_clock = clock() + CLOCKS_PER_SEC*max_time;
+        start_clock = clock();
     }
     HfstOneLevelPaths * results = new HfstOneLevelPaths;
     lookup_paths = results;
@@ -482,7 +493,7 @@ void Transducer::get_analyses(unsigned int input_pos,
     }
     if (max_time > 0.0) {
         // quit if we've overspent our time
-        if (clock() > max_clock) {
+        if ((((double) clock() - start_clock) / CLOCKS_PER_SEC) > max_time) {
             return;
         }
     }
