@@ -1,15 +1,12 @@
 // -*- mode: c++; -*-
-//       This program is free software: you can redistribute it and/or modify
-//       it under the terms of the GNU General Public License as published by
-//       the Free Software Foundation, version 3 of the License.
-//
-//       This program is distributed in the hope that it will be useful,
-//       but WITHOUT ANY WARRANTY; without even the implied warranty of
-//       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//       GNU General Public License for more details.
-//
-//       You should have received a copy of the GNU General Public License
-//       along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// Copyright (c) 2016 University of Helsinki                          
+//                                                                    
+// This library is free software; you can redistribute it and/or      
+// modify it under the terms of the GNU Lesser General Public         
+// License as published by the Free Software Foundation; either       
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more      
+// information.
 
 #ifndef _HFST_OL_TRANSDUCER_TRANSDUCER_H_
 #define _HFST_OL_TRANSDUCER_TRANSDUCER_H_
@@ -658,6 +655,8 @@ public:
             return (i < TRANSITION_TARGET_TABLE_START) ? 
                 table[i] : table[i-TRANSITION_TARGET_TABLE_START];
         }
+
+    std::vector<T> get_vector(void) const { return std::vector<T>(table); } ;
   
     void display(bool transition_table) const
         {
@@ -746,7 +745,6 @@ public:
         { return index_table[i].final(); }
     Weight get_final_weight(TransitionTableIndex i) const
         { return index_table[i].final_weight(); }
-
   
     void display() const
         {
@@ -857,7 +855,7 @@ protected:
     ssize_t max_lookups;
     unsigned int recursion_depth_left;
     double max_time;
-    clock_t max_clock;
+    clock_t start_clock;
 
     void try_epsilon_transitions(unsigned int input_tape_pos,
                                  unsigned int output_tape_pos,
@@ -925,11 +923,11 @@ public:
     const SymbolTable& get_symbol_table() const
         { return alphabet->get_symbol_table(); }
 
-
     const TransitionIndex& get_index(TransitionTableIndex i) const
         { return tables->get_index(i); }
     const Transition& get_transition(TransitionTableIndex i) const
         { return tables->get_transition(i); }
+    
     bool final_index(TransitionTableIndex i) const
         {
             if (indexes_transition_table(i)) {
@@ -950,7 +948,7 @@ public:
     TransducerTable<TransitionW> copy_transitionw_table();
     TransducerTable<TransitionIndex> copy_index_table();
     TransducerTable<Transition> copy_transition_table();
-
+    
     // state_index must be an index to a state which is defined as either:
     // (1) the start of a set of entries in the transition index table, or
     // (2) the boundary before a set of entries in the transition table, in
@@ -962,6 +960,7 @@ public:
 
 
     bool initialize_input(const char * input_str);
+    void include_symbol_in_alphabet(const std::string & sym);
     HfstOneLevelPaths * lookup_fd(const StringVector & s, ssize_t limit = -1,
         double time_cutoff = 0.0);
     /* Tokenize and lookup, accounting for flag diacritics, the surface string
