@@ -1,20 +1,16 @@
+// Copyright (c) 2016 University of Helsinki                          
+//                                                                    
+// This library is free software; you can redistribute it and/or      
+// modify it under the terms of the GNU Lesser General Public         
+// License as published by the Free Software Foundation; either       
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more      
+// information.
+
 //! @file LexcCompiler.cc
 //!
 //! @brief Implementation of lexc compilation.
 
-
-//       This program is free software: you can redistribute it and/or modify
-//       it under the terms of the GNU General Public License as published by
-//       the Free Software Foundation, version 3 of the License.
-//
-//       This program is distributed in the hope that it will be useful,
-//       but WITHOUT ANY WARRANTY; without even the implied warranty of
-//       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//       GNU General Public License for more details.
-//
-//       You should have received a copy of the GNU General Public License
-//       along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
 #if HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -37,13 +33,19 @@ using std::set_difference;
 #include "HfstTransducer.h"
 #include "XreCompiler.h"
 #include "lexc-utils.h"
-#include "lexc-parser.hh"
+
+#ifdef YACC_USE_PARSER_H_EXTENSION
+  #include "lexc-parser.h"
+#else
+  #include "lexc-parser.hh"
+#endif
+
 #include "xre_utils.h"
 #include "HfstSymbolDefs.h"
 //#include "../../../tools/src/HfstStrings2FstTokenizer.h"
 
 #ifdef WINDOWS
-#include "../../../tools/src/hfst-string-conversions.h"
+#include "hfst-string-conversions.h"
 using hfst::hfst_fprintf_console;
 #endif // WINDOWS
 
