@@ -1,3 +1,12 @@
+// Copyright (c) 2016 University of Helsinki                          
+//                                                                    
+// This library is free software; you can redistribute it and/or      
+// modify it under the terms of the GNU Lesser General Public         
+// License as published by the Free Software Foundation; either       
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more      
+// information.
+
 //! @file XfstCompiler.h
 //!
 //! @brief A class that encapsulates compilation of Xerox fst language scripts
@@ -9,18 +18,6 @@
 //!        This class is merely a wrapper around lex and yacc functions handling
 //!        the parsing.
 
-//       This program is free software: you can redistribute it and/or modify
-//       it under the terms of the GNU General Public License as published by
-//       the Free Software Foundation, version 3 of the License.
-//
-//       This program is distributed in the hope that it will be useful,
-//       but WITHOUT ANY WARRANTY; without even the implied warranty of
-//       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//       GNU General Public License for more details.
-//
-//       You should have received a copy of the GNU General Public License
-//       along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
 #ifndef GUARD_XfstCompiler_h
 #define GUARD_XfstCompiler_h
 
@@ -688,8 +685,8 @@ class XfstCompiler
   XfstCompiler& print_transducer_info();
   XfstCompiler& add_prop_line(char* line);
 
-  XfstCompiler& apply_line(char* line, const HfstTransducer * t, size_t cutoff);
-  XfstCompiler& apply_line(char* line, HfstBasicTransducer * t);
+  XfstCompiler& lookup(char* line, const HfstTransducer * t, size_t cutoff);
+  XfstCompiler& lookup(char* line, HfstBasicTransducer * t);
 
   XfstCompiler& apply_up_line(char* line);
   XfstCompiler& apply_down_line(char* line);
