@@ -1,3 +1,12 @@
+// Copyright (c) 2016 University of Helsinki                          
+//                                                                    
+// This library is free software; you can redistribute it and/or      
+// modify it under the terms of the GNU Lesser General Public         
+// License as published by the Free Software Foundation; either       
+// version 3 of the License, or (at your option) any later version.
+// See the file COPYING included with this distribution for more      
+// information.
+
 /**
  * @file xre.cc
  *
@@ -1033,9 +1042,12 @@ void warn_about_hfst_special_symbol(const char * symbol)
     return;
   if (!verbose_)
     return;
-  std::ostream * err = xreerrstr();
-  *err << "warning: '" << symbol << "' is not an ordinary symbol in hfst" << std::endl;
-  xreflush(err);
+  if (hfst::xre::verbose_)
+    {
+      std::ostream * err = xreerrstr();
+      *err << "warning: '" << symbol << "' is not an ordinary symbol in hfst" << std::endl;
+      xreflush(err);
+    }
 }
 
 void warn_about_special_symbols_in_replace(HfstTransducer * t)
