@@ -866,7 +866,7 @@ print_version()
 {
   // c.f. http://www.gnu.org/prep/standards/standards.html#g_t_002d_002dversion
     fprintf(message_out, "%s %s (" PACKAGE_STRING ")\n"
-             "Copyright (C) 2010 University of Helsinki,\n"
+             "Copyright (C) 2016 University of Helsinki,\n"
              "License GPLv3: GNU GPL version 3 "
              "<http://gnu.org/licenses/gpl.html>\n"
              "This is free software: you are free to change and "
@@ -880,7 +880,7 @@ print_report_bugs()
 {
   fprintf(message_out, "Report bugs to <" PACKAGE_BUGREPORT "> "
           "or directly to our bug tracker at:\n"
-          "<https://sourceforge.net/tracker/?atid=1061990&group_id=224521&func=browse>\n");
+          "<https://github.com/hfst/hfst/issues>\n");
 }
 
 
