@@ -230,6 +230,9 @@ print_usage()
     // c.f. http://www.gnu.org/prep/standards/standards.html#g_t_002d_002dhelp
     fprintf(message_out, "Usage: %s [OPTIONS...] [INFILE]\n"
            "perform transducer lookup (apply)\n"
+           "NOTE: hfst-lookup does lookup from left to right as opposed to xfst and foma\n"
+           "      lookup which is carried out from right to left. In order to do lookup\n"
+           "      in a similar way as xfst and foma, use 'hfst-flookup' instead.\n"
         "\n", program_name);
 
     print_common_program_options(message_out);
@@ -1116,7 +1119,7 @@ HfstOneLevelPaths*
 lookup_simple(const HfstOneLevelPath& s, HfstTransducer& t, bool* infinity)
 {
   HfstOneLevelPaths* results = 0;
-  if (t.is_lookup_infinitely_ambiguous(s.second))
+  if (time_cutoff == 0.0 && t.is_lookup_infinitely_ambiguous(s.second))
     {
       if (!silent && infinite_cutoff > 0) {
     warning(0, 0, "Got infinite results, number of cycles limited to " SIZE_T_SPECIFIER "",
