@@ -48,6 +48,9 @@ bool print_usage(void)
     "\n" <<
     "Usage: " << PACKAGE_NAME << " [OPTIONS] TRANSDUCER\n" <<
     "Run a transducer on standard input (one word per line) and print analyses\n" <<
+    "NOTE: hfst-optimized-lookup does lookup from left to right as opposed to xfst\n" <<
+    "      and foma lookup which is carried out from right to left. In order to do\n" <<
+    "      lookup in a similar way as xfst and foma, invert the transducer first.\n" <<
     "\n" <<
     "  -h, --help                  Print this help message\n" <<
     "  -V, --version               Print version information\n" <<
@@ -134,7 +137,7 @@ int main(int argc, char **argv)
         };
       
       int option_index = 0;
-      c = getopt_long(argc, argv, "hVvqsewb:uxfn:p::", long_options, &option_index);
+      c = getopt_long(argc, argv, "hVvqsewb:t:uxfn:p::", long_options, &option_index);
 
       if (c == -1) // no more options to look at
         break;
@@ -467,10 +470,6 @@ void runTransducer (genericTransducer T)
   *str = 0;
   char * old_str = str;
 
-  if (time_cutoff > 0.0) {
-      max_clock = clock() + CLOCKS_PER_SEC*time_cutoff;
-  }
-        
   while(true)
     {
 #ifdef WINDOWS
@@ -520,7 +519,6 @@ void runTransducer (genericTransducer T)
           input_string[i] = k;
           ++i;
         }
-
       str = old_str;
       if (failed)
         { // tokenization failed
@@ -528,23 +526,29 @@ void runTransducer (genericTransducer T)
             {
 #ifdef WINDOWS
           if (!pipe_output)
-              hfst_fprintf_console(stdout, "%s\t+?\n", str);
+              hfst_fprintf_console(stdout, "%s\t+?\n\n", str);
           else
 #endif
-            std::cout << str << "\t+?" << std::endl;
+              std::cout << str << "\t+?" << std::endl << std::endl;
 
 #ifdef WINDOWS
           if (!pipe_output)
-            hfst_fprintf_console(stdout, "\n");
+            hfst_fprintf_console(stdout, "\n\n");
           else
 #endif
-            std::cout << std::endl;
+              std::cout << std::endl << std::endl;
             }
           continue;
         }
 
       input_string[i] = NO_SYMBOL_NUMBER;
 
+      if (time_cutoff > 0.0) {
+          start_clock = clock();
+          call_counter = 0;
+          limit_reached = false;
+      }
+      
       T.analyze(input_string);
       T.printAnalyses(std::string(str));
     }
@@ -997,12 +1001,17 @@ void Transducer::get_analyses(SymbolNumber * input_symbol,
                               SymbolNumber * original_output_string,
                               TransitionTableIndex i)
 {
-  if (time_cutoff > 0.0) {
-    // quit if we've overspent our time
-    if (clock() > max_clock) {
-      return;
+    if (time_cutoff > 0.0) {
+        // Check to see if time has been overspent. For speed, only check every
+        // million calls and set a flag.
+        ++call_counter;
+        if (limit_reached ||
+            (call_counter % 1000000 == 0 &&
+             ((((double) clock() - start_clock) / CLOCKS_PER_SEC) > time_cutoff))) {
+            limit_reached = true;
+            return;
+        }
     }
-  }
 #if OL_FULL_DEBUG
   std::cout << "get_analyses " << i << std::endl;
 #endif
@@ -1081,17 +1090,17 @@ void Transducer::printAnalyses(std::string prepend)
         {
 #ifdef WINDOWS
           if (!pipe_output)
-            hfst_fprintf_console(stdout, "%s\t+?\n", prepend.c_str());
+            hfst_fprintf_console(stdout, "%s\t+?\n\n", prepend.c_str());
           else
 #endif
-            std::cout << prepend << "\t+?" << std::endl;
+          std::cout << prepend << "\t+?" << std::endl << std::endl;
 
 #ifdef WINDOWS
           if (!pipe_output)
-            hfst_fprintf_console(stdout, "\n");
+            hfst_fprintf_console(stdout, "\n\n");
           else
 #endif
-            std::cout << std::endl;
+          std::cout << std::endl << std::endl;
           return;
         }
       int i = 0;
@@ -1109,7 +1118,7 @@ void Transducer::printAnalyses(std::string prepend)
             }                   
 
 #ifdef WINDOWS
-          if (!pipe_mode)
+          if (!pipe_output)
             hfst_fprintf_console(stdout, "%s\n", it->c_str());
           else
 #endif
@@ -1121,7 +1130,7 @@ void Transducer::printAnalyses(std::string prepend)
       display_vector.clear(); // purge the display vector
 
 #ifdef WINDOWS
-      if (!pipe_mode)
+      if (!pipe_output)
         hfst_fprintf_console(stdout, "\n");
       else
 #endif
@@ -1136,18 +1145,18 @@ void TransducerUniq::printAnalyses(std::string prepend)
     {
 
 #ifdef WINDOWS
-      if (!pipe_mode)
-        hfst_fprintf_console(stdout, "%s\t+?\n", prepend.c_str());
+      if (!pipe_output)
+        hfst_fprintf_console(stdout, "%s\t+?\n\n", prepend.c_str());
       else
 #endif
-        std::cout << prepend << "\t+?" << std::endl;
+        std::cout << prepend << "\t+?" << std::endl << std::endl;
 
 #ifdef WINDOWS
-      if (!pipe_mode)
-        hfst_fprintf_console("\n");
+      if (!pipe_output)
+        hfst_fprintf_console(stdout, "\n\n");
       else
 #endif
-        std::cout << std::endl;
+        std::cout << std::endl << std::endl;
 
       return;
     }
@@ -1158,7 +1167,7 @@ void TransducerUniq::printAnalyses(std::string prepend)
       if (outputType == xerox)
         {
 #ifdef WINDOWS
-      if (!pipe_mode)
+      if (!pipe_output)
         hfst_fprintf_console(stdout, "%s\t", prepend.c_str());
       else
 #endif
@@ -1166,7 +1175,7 @@ void TransducerUniq::printAnalyses(std::string prepend)
         }
 
 #ifdef WINDOWS
-      if (!pipe_mode)
+      if (!pipe_output)
         hfst_fprintf_console(stdout, "%s\n", it->c_str());
       else
 #endif
@@ -1177,8 +1186,8 @@ void TransducerUniq::printAnalyses(std::string prepend)
     }
   display_vector.clear(); // purge the display set
 #ifdef WINDOWS
-  if (!pipe_mode)
-    hfst_fprintf_console("\n");
+  if (!pipe_output)
+    hfst_fprintf_console(stdout, "\n");
   else
 #endif
     std::cout << std::endl;
@@ -1189,19 +1198,18 @@ void TransducerFdUniq::printAnalyses(std::string prepend)
   if (outputType == xerox && display_vector.size() == 0)
     {
 #ifdef WINDOWS
-  if (!pipe_mode)
-    hfst_fprintf_console(stdout, "%s\t+?\n", prepend.c_str());
+  if (!pipe_output)
+    hfst_fprintf_console(stdout, "%s\t+?\n\n", prepend.c_str());
   else
 #endif
-    std::cout << prepend << "\t+?" << std::endl;
+      std::cout << prepend << "\t+?" << std::endl << std::endl;
 
 #ifdef WINDOWS
-  if (!pipe_mode)
-    hfst_fprintf_console(stdout, "\n");
+  if (!pipe_output)
+    hfst_fprintf_console(stdout, "\n\n");
   else
 #endif
-    std::cout << std::endl;
-
+      std::cout << std::endl << std::endl;
   return;
     }
   int i = 0;
@@ -1211,7 +1219,7 @@ void TransducerFdUniq::printAnalyses(std::string prepend)
       if (outputType == xerox)
         {
 #ifdef WINDOWS
-          if (!pipe_mode)
+          if (!pipe_output)
             hfst_fprintf_console(stdout, "%s\t", prepend.c_str());
           else
 #endif
@@ -1219,7 +1227,7 @@ void TransducerFdUniq::printAnalyses(std::string prepend)
         }
 
 #ifdef WINDOWS
-      if (!pipe_mode)
+      if (!pipe_output)
         hfst_fprintf_console(stdout, "%s\n", it->c_str());
       else
 #endif
@@ -1231,8 +1239,8 @@ void TransducerFdUniq::printAnalyses(std::string prepend)
   display_vector.clear(); // purge the display set
 
 #ifdef WINDOWS
-  if (!pipe_mode)
-    hfst_fprintf_console("\n");
+  if (!pipe_output)
+    hfst_fprintf_console(stdout, "\n");
   else
 #endif
     std::cout << std::endl;
@@ -1628,16 +1636,16 @@ void TransducerW::printAnalyses(std::string prepend)
     {
 #ifdef WINDOWS
       if (!pipe_output)
-        hfst_fprintf_console(stdout, "%s\t+?\n", prepend.c_str());
+        hfst_fprintf_console(stdout, "%s\t+?\n\n", prepend.c_str());
       else
 #endif
-        std::cout << prepend << "\t+?" << std::endl;
+          std::cout << prepend << "\t+?" << std::endl << std::endl;
 
 #ifdef WINDOWS
       if (!pipe_output)
-        hfst_fprintf_console(stdout, "\n");
+        hfst_fprintf_console(stdout, "\n\n");
       else
-        std::cout << std::endl;
+          std::cout << std::endl << std::endl;
 #endif
 
       return;
@@ -1757,7 +1765,7 @@ void TransducerWUniq::printAnalyses(std::string prepend)
         {
 #ifdef WINDOWS
           if (!pipe_output)
-            hfst_fprintf_console("\t%f", (*display_it).first);
+            hfst_fprintf_console(stdout, "\t%f", (*display_it).first);
           else
 #endif
             std::cout << '\t' << (*display_it).first;
@@ -1831,7 +1839,7 @@ void TransducerWFdUniq::printAnalyses(std::string prepend)
 
 #ifdef WINDOWS
       if (!pipe_output)
-        hfst_fprintf_console("%s", (*display_it).second);
+        hfst_fprintf_console(stdout, "%s", (*display_it).second);
       else
 #endif
         std::cout << (*display_it).second;
@@ -1839,16 +1847,16 @@ void TransducerWFdUniq::printAnalyses(std::string prepend)
       if (displayWeightsFlag)
         {
 #ifdef WINDOWS
-          if (!pipe_mode)
-            hfst_fprintf("\t%f", (*display_it).first);
+          if (!pipe_output)
+            hfst_fprintf_console(stdout, "\t%f", (*display_it).first);
           else
 #endif
             std::cout << '\t' << (*display_it).first;
         }
 
 #ifdef WINDOWS
-          if (!pipe_mode)
-            hfst_fprintf_console("\n");
+          if (!pipe_output)
+            hfst_fprintf_console(stdout, "\n");
           else
 #endif
             std::cout << std::endl;
@@ -1856,8 +1864,8 @@ void TransducerWFdUniq::printAnalyses(std::string prepend)
   display_map.clear();
 
 #ifdef WINDOWS
-  if (!pipe_mode)
-    hfst_fprintf_console("\n");
+  if (!pipe_output)
+    hfst_fprintf_console(stdout, "\n");
   else
 #endif
     std::cout << std::endl;
@@ -1871,6 +1879,17 @@ void TransducerW::get_analyses(SymbolNumber * input_symbol,
 #if OL_FULL_DEBUG
   std::cerr << "get analyses " << i << " " << current_weight << std::endl;
 #endif
+  if (time_cutoff > 0.0) {
+      // Check to see if time has been overspent. For speed, only check every
+      // million calls and set a flag.
+      ++call_counter;
+      if (limit_reached ||
+          (call_counter % 1000000 == 0 &&
+           ((((double) clock() - start_clock) / CLOCKS_PER_SEC) > time_cutoff))) {
+          limit_reached = true;
+          return;
+      }
+  }
   if (i >= TRANSITION_TARGET_TABLE_START )
     {
       i -= TRANSITION_TARGET_TABLE_START;
