@@ -60,8 +60,10 @@ bool echoInputsFlag = false;
 bool beFast = false;
 int maxAnalyses = INT_MAX;
 bool preserveDiacriticRepresentationsFlag = false;
+bool limit_reached = false;
+unsigned long call_counter = 0;
 double time_cutoff = 0.0;
-clock_t max_clock;
+clock_t start_clock;
 
 #define MAX_IO_STRING 5000
 
