@@ -131,17 +131,17 @@ print_usage()
     fprintf(message_out,
         "In Xerox mode, the input should be a twolc file. Tests consist of\n"
         "two lines: an input form and an output form. The test cases are\n"
-        "specialized comments prefixed with either '!€' or '!$' depeding on\n"
+        "specialized comments prefixed with either '!!€' or '!!$' depeding on\n"
         "whether the pair should succeed or fail. An example of a positive\n"
         "test:\n\n"
 
-        "!€ earlYer\n"
-        "!€ earlier\n\n"
+        "!!€ earlYer\n"
+        "!!€ earlier\n\n"
 
         "An example of a negative test:\n\n"
 
-        "!$ earlYer\n"
-        "!$ earlyer\n");
+        "!!$ earlYer\n"
+        "!!$ earlyer\n");
     fprintf(message_out, "\n");
     fprintf(message_out,
         "In silent mode (-s), the program won't print anything. Only the\n"
@@ -172,7 +172,7 @@ parse_options(int argc, char** argv)
         int option_index = 0;
         // add tool-specific options here 
         char c = getopt_long(argc, argv, HFST_GETOPT_COMMON_SHORT
-                             HFST_GETOPT_UNARY_SHORT "I:Nx",
+                             HFST_GETOPT_UNARY_SHORT "I:NX",
                              long_options, &option_index);
         if (-1 == c)
         {
@@ -216,6 +216,31 @@ parse_options(int argc, char** argv)
     return EXIT_CONTINUE;
 }
 
+std::string &replace_all_substr(const std::string &substr, 
+                                const std::string &repl,
+                                std::string &str)
+{
+  int pos = 0;
+  while ((pos = str.find(substr, pos)) != std::string::npos)
+    {
+      str.replace(pos, substr.size(), repl);
+    }
+  return str;
+}
+#define PTPP "PAIR_TEST_PERC_PERC"
+#define PTPC "PAIR_TEST_PERC_COL"
+// perc_escaped is a string where special symols are escaped using
+// %. Transform it into a string where specail symbols are escaped
+// using \. 
+std::string backslash_escape(std::string perc_escaped)
+{
+  replace_all_substr("%%", PTPP, perc_escaped);
+  replace_all_substr("%:", PTPC, perc_escaped);
+  replace_all_substr("%", "", perc_escaped);
+  replace_all_substr(PTPC, "\\:", perc_escaped);
+  return replace_all_substr(PTPP, "%", perc_escaped);
+}
+
 HfstState get_target(const std::string &isymbol,
              const std::string &osymbol,
              HfstState s,const HfstBasicTransducer &t,
@@ -467,10 +492,10 @@ std::string strip_space(const std::string &line)
 }
 
 bool is_positive_test_line(const std::string &line)
-{ return strip_space(line).substr(0,sizeof("!€") - 1) == "!€"; }
+{ return strip_space(line).substr(0,sizeof("!!€") - 1) == "!!€"; }
 
 bool is_negative_test_line(const std::string &line)
-{ return strip_space(line).substr(0,sizeof("!$") - 1) == "!$"; }
+{ return strip_space(line).substr(0,sizeof("!!$") - 1) == "!!$"; }
 
 int
 process_stream(HfstInputStream& inputstream, FILE* outstream)
@@ -583,8 +608,8 @@ process_stream(HfstInputStream& inputstream, FILE* outstream)
         /*
           Read test cases from a twolc source file.
 
-          Positive test cases are prefixed by "!€" and negative test
-          cases by "!$".
+          Positive test cases are prefixed by "!!€" and negative test
+          cases by "!!$".
 
           Each test case spans two lines: the input and output cases.
          */
@@ -613,8 +638,8 @@ process_stream(HfstInputStream& inputstream, FILE* outstream)
 
             if (is_positive_test_line(line))
               {
-                // "!€ xyz" -> "xyz"
-                std::string test_case = strip_space(line).substr(sizeof("!€") - 1);
+                // "!!€ xyz" -> "xyz"
+                std::string test_case = strip_space(line).substr(sizeof("!!€") - 1);
                 test_case = strip_space(test_case);
 
                 positive_test_cases.push_back(test_case);
@@ -623,8 +648,8 @@ process_stream(HfstInputStream& inputstream, FILE* outstream)
               }
             else if (is_negative_test_line(line))
               {
-                // "!$ xyz" -> "xyz"
-                std::string test_case = strip_space(line).substr(sizeof("!$") - 1);
+                // "!!$ xyz" -> "xyz"
+                std::string test_case = strip_space(line).substr(sizeof("!!$") - 1);
                 test_case = strip_space(test_case);
                 
                 negative_test_cases.push_back(test_case);
@@ -657,9 +682,30 @@ process_stream(HfstInputStream& inputstream, FILE* outstream)
             const std::string &input_case = positive_test_cases[i];
             const std::string &output_case = positive_test_cases[i + 1];
 
-            StringPairVector test_case = input_tokenizer.tokenize_string_pair
-              (input_case + ":" + output_case, false);
-            
+            StringPairVector test_case;
+            try 
+              {
+                // We need to convert the %-escaped input and output
+                // string to \-escpaed strings for input_toknizer.
+                test_case = input_tokenizer.tokenize_string_pair
+                  (backslash_escape(input_case) + ":" + 
+                   backslash_escape(output_case), false);
+                test_case.insert
+                  (test_case.begin(),
+                   StringPair("@#@",hfst::internal_epsilon));
+                test_case.insert
+                  (test_case.end(),
+                   StringPair("@#@",hfst::internal_epsilon));
+
+              }
+            catch (const hfst::UnescapedColsFound &e)
+              {
+                error(EXIT_FAILURE, 0, 
+                      "The correspondence %s %s contains unescaped "
+                      "colon-symbols. Escape them using %%.",
+                      input_case.c_str(), output_case.c_str(), line);
+              }
+
             int new_exit_code = test(test_case,
                                      input_case + " : " + output_case,
                                      grammar,
@@ -677,9 +723,29 @@ process_stream(HfstInputStream& inputstream, FILE* outstream)
             const std::string &input_case = negative_test_cases[i];
             const std::string &output_case = negative_test_cases[i + 1];
 
-            StringPairVector test_case = input_tokenizer.tokenize_string_pair
-              (input_case + ":" + output_case, false);
-            
+            StringPairVector test_case;
+            try
+              {
+                // We need to convert the %-escaped input and output
+                // string to \-escpaed strings for input_toknizer.
+                test_case = input_tokenizer.tokenize_string_pair
+                  (backslash_escape(input_case) + ":" + 
+                   backslash_escape(output_case), false);
+                test_case.insert
+                  (test_case.begin(),
+                   StringPair("@#@",hfst::internal_epsilon));
+                test_case.insert
+                  (test_case.end(),
+                   StringPair("@#@",hfst::internal_epsilon));
+              }
+            catch (const hfst::UnescapedColsFound &e)
+              {
+                error(EXIT_FAILURE, 0, 
+                      "The correspondence %s %s contains unquoted "
+                      "colon-symbols. Quote them using %%.",
+                      input_case.c_str(), output_case.c_str(), line);
+              }
+
             int new_exit_code = test(test_case,
                                      input_case + " : " + output_case,
                                      grammar,
