@@ -190,79 +190,77 @@ process_stream(HfstOutputStream& outstream)
         std::cerr << "Building hfst-ol alphabet... ";
     }
     
-    if (definitions.size() > 1) {
-        
-        // A dummy transducer with an alphabet with all the symbols
-        HfstTransducer harmonizer(compilation_format);
-        
-        // First we need to collect a unified alphabet from all the transducers.
-        hfst::StringSet symbols_seen;
-        for (std::map<std::string, HfstTransducer *>::const_iterator it =
-                 definitions.begin(); it != definitions.end(); ++it) {
-            hfst::StringSet string_set = it->second->get_alphabet();
-            for (hfst::StringSet::const_iterator sym = string_set.begin();
-                 sym != string_set.end(); ++sym) {
-                if (symbols_seen.count(*sym) == 0) {
-                    harmonizer.disjunct(HfstTransducer(*sym, compilation_format));
-                    symbols_seen.insert(*sym);
-                }
+    // A dummy transducer with an alphabet with all the symbols
+    HfstTransducer harmonizer(compilation_format);
+    
+    // First we need to collect a unified alphabet from all the transducers.
+    hfst::StringSet symbols_seen;
+    for (std::map<std::string, HfstTransducer *>::const_iterator it =
+             definitions.begin(); it != definitions.end(); ++it) {
+        hfst::StringSet string_set = it->second->get_alphabet();
+        for (hfst::StringSet::const_iterator sym = string_set.begin();
+             sym != string_set.end(); ++sym) {
+            if (symbols_seen.count(*sym) == 0) {
+                harmonizer.disjunct(HfstTransducer(*sym, compilation_format));
+                symbols_seen.insert(*sym);
             }
         }
-        if (symbols_seen.size() == 0) {
-            // We don't recognise anything, go home early
-            std::cerr << program_name << ": Empty ruleset, nothing to write\n";
-            return EXIT_FAILURE;
-        }
-        
-        // Then we convert it...
-        harmonizer.convert(hfst::HFST_OLW_TYPE);
-        // Use these for naughty intermediate steps to make sure
-        // everything has the same alphabet
-        hfst::HfstBasicTransducer * intermediate_tmp;
-        hfst_ol::Transducer * harmonized_tmp;
-        hfst::HfstTransducer * output_tmp;
-        
-        if (verbose) {
-            double duration = (clock() - timer) /
-                (double) CLOCKS_PER_SEC;
-            timer = clock();
-            std::cerr << "built in " << duration << " seconds\n";
-            std::cerr << "Converting TOP... ";
-        }
+    }
+    if (symbols_seen.size() == 0) {
+        // We don't recognise anything, go home early
+        std::cerr << program_name << ": Empty ruleset, nothing to write\n";
+        return EXIT_FAILURE;
+    }
+    
+    // Then we convert it...
+    harmonizer.convert(hfst::HFST_OLW_TYPE);
+    // Use these for naughty intermediate steps to make sure
+    // everything has the same alphabet
+    hfst::HfstBasicTransducer * intermediate_tmp;
+    hfst_ol::Transducer * harmonized_tmp;
+    hfst::HfstTransducer * output_tmp;
+    
+    if (verbose) {
+        double duration = (clock() - timer) /
+            (double) CLOCKS_PER_SEC;
+        timer = clock();
+        std::cerr << "built in " << duration << " seconds\n";
+        std::cerr << "Converting TOP... ";
+    }
         
     // When done compiling everything, look for TOP and output it first.
-        if (definitions.count("TOP") == 1) {
-            intermediate_tmp = hfst::implementations::ConversionFunctions::
-                hfst_transducer_to_hfst_basic_transducer(*definitions["TOP"]);
-            harmonized_tmp = hfst::implementations::ConversionFunctions::
-                hfst_basic_transducer_to_hfst_ol(intermediate_tmp,
-                                                 true, // weighted
-                                                 "", // no special options
-                                                 &harmonizer); // harmonize with this
-            output_tmp = hfst::implementations::ConversionFunctions::
-                hfst_ol_to_hfst_transducer(harmonized_tmp);
-            output_tmp->set_name("TOP");
-            outstream << *output_tmp;
-            delete definitions["TOP"];
-            definitions.erase("TOP");
-            delete intermediate_tmp;
-            delete output_tmp;
-        }
+    if (definitions.count("TOP") == 1) {
+        intermediate_tmp = hfst::implementations::ConversionFunctions::
+            hfst_transducer_to_hfst_basic_transducer(*definitions["TOP"]);
+        harmonized_tmp = hfst::implementations::ConversionFunctions::
+            hfst_basic_transducer_to_hfst_ol(intermediate_tmp,
+                                             true, // weighted
+                                             "", // no special options
+                                             &harmonizer); // harmonize with this
+        output_tmp = hfst::implementations::ConversionFunctions::
+            hfst_ol_to_hfst_transducer(harmonized_tmp);
+        output_tmp->set_name("TOP");
+        outstream << *output_tmp;
+        delete definitions["TOP"];
+        definitions.erase("TOP");
+        delete intermediate_tmp;
+        delete output_tmp;
+
         if (verbose) {
             double duration = (clock() - timer) /
                 (double) CLOCKS_PER_SEC;
             timer = clock();
             std::cerr << "converted in " << duration << " seconds\n";
         }
-        
+    
         for (std::map<std::string, HfstTransducer *>::iterator it =
                  definitions.begin(); it != definitions.end(); ++it) {
             if (verbose) {
                 std::cerr << "Converting " << it->first << "... ";
                 timer = clock();
-        }
+            }
             intermediate_tmp = hfst::implementations::ConversionFunctions::
-            hfst_transducer_to_hfst_basic_transducer(*(it->second));
+                hfst_transducer_to_hfst_basic_transducer(*(it->second));
             harmonized_tmp = hfst::implementations::ConversionFunctions::
                 hfst_basic_transducer_to_hfst_ol(intermediate_tmp,
                                                  true, // weighted
@@ -280,33 +278,11 @@ process_stream(HfstOutputStream& outstream)
                     (double) CLOCKS_PER_SEC;
                 std::cerr << "converted in " << duration << " seconds\n";
             }
-    }
-    } else if (definitions.size() == 1) {
-        // A common case of only one top-level definition
-        if (verbose) {
-            double duration = (clock() - timer) /
-                (double) CLOCKS_PER_SEC;
-            timer = clock();
-            std::cerr << "built in " << duration << " seconds\n";
-            std::cerr << "Converting TOP... ";
         }
-        
-        std::map<std::string, HfstTransducer *>::iterator only_entry = definitions.begin();
-        only_entry->second->convert(hfst::HFST_OLW_TYPE);
-        
-        if (verbose) {
-            double duration = (clock() - timer) /
-                (double) CLOCKS_PER_SEC;
-            timer = clock();
-            std::cerr << "converted in " << duration << " seconds\n";
-        }
-        
-        outstream << *(only_entry->second);
     } else {
         std::cerr << program_name << ": Empty ruleset, nothing to write\n";
         return EXIT_FAILURE;
     }
-    
     return EXIT_SUCCESS;
 }
 
